/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.mycompany.ostracraft;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;

/**
 *
 * @author Jack
 */
public class AccountDatabase {
    
    private static String path = App.rootPath + "data/accounts.txt";
    
    public static AccountDatabase database = new AccountDatabase();
    
    private RandomAccessFile file = null;
    private int entrySize = 1000;
    
    public AccountDatabase () {
        openFile();
    }
    
    private void openFile() {
    	try {
            file = new RandomAccessFile(path, "rw");
        } catch (FileNotFoundException exception) {
            System.out.println("Unable to open file \"" + path + "\".");
        }
    }
    
    private void flush() {
    	try {
    		file.close();
    	} catch (IOException exception) {
		    System.out.println("Unable to access file \"" + path + "\".");
		}
    	openFile();
    }
    
    private int getEntryCount() {
        try {
            return (int)(file.length() / entrySize);
        } catch (IOException exception) {
            System.out.println("Unable to access file \"" + path + "\".");
            return 0;
        }
    }
    
    private String getEntry(int index) {
        try {
            byte[] tempBuffer = new byte[entrySize];
            file.seek((long)index * (long)entrySize);
            file.read(tempBuffer, 0, entrySize);
            return new String(tempBuffer, "UTF-8");
        } catch (IOException exception) {
            System.out.println("Unable to access file \"" + path + "\".");
            return "";
        }
    }
    
    private void setEntry(int index, String entry) {
        try {
            byte[] tempBuffer = new byte[entrySize];
            byte[] tempBuffer2 = entry.getBytes();
            int tempIndex = 0;
            while (tempIndex < tempBuffer2.length) {
                byte tempByte = tempBuffer2[tempIndex];
                tempBuffer[tempIndex] = tempByte;
                tempIndex += 1;
            }
            while (tempIndex < tempBuffer.length) {
                tempBuffer[tempIndex] = ' ';
                tempIndex += 1;
            }
            file.seek((long)index * (long)entrySize);
            file.write(tempBuffer, 0, entrySize);
            flush();
        } catch (IOException exception) {
            System.out.println("Unable to access file \"" + path + "\".");
        }
    }
    
    private Account convertEntryToAccount(String entry) {
        entry = entry.substring(1, entry.indexOf("]"));
        String[] tempList = entry.split(";");
        if (tempList.length < 4) {
            return null;
        }
        int index = 0;
        while (index < tempList.length) {
            String tempTerm = tempList[index];
            tempTerm = StringEncoder.encoder.decode(tempTerm);
            tempList[index] = tempTerm;
            index += 1;
        }
        int tempId = Integer.parseInt(tempList[0]);
        String tempName = tempList[1];
        String tempPasswordHash = tempList[2];
        String tempEmail = tempList[3];
        return new Account(tempId, tempName, tempPasswordHash, tempEmail);
    }
    
    private String convertAccountToEntry(Account account) {
        String[] tempList = {
            Integer.toString(account.getId()),
            account.getName(),
            account.getPasswordHash(),
            account.getEmail()
        };
        int index = 0;
        while (index < tempList.length) {
            String tempTerm = tempList[index];
            tempTerm = StringEncoder.encoder.encode(tempTerm);
            tempList[index] = tempTerm;
            index += 1;
        }
        return "[" + String.join(";", tempList) + "]";
    }
    
    private Account getAccount(int index) {
        String tempEntry = getEntry(index);
        return convertEntryToAccount(tempEntry);
    }
    
    private void setAccount(int index, Account account) {
        String tempEntry = convertAccountToEntry(account);
        setEntry(index, tempEntry);
    }
    
    private int findAccountById(int id) {
        int entryCount = getEntryCount();
        int output = 0;
        while (output < entryCount) {
            Account tempAccount = getAccount(output);
            if (tempAccount.getId() == id) {
                return output;
            }
            output += 1;
        }
        return -1;
    }
    
    private int findAccountByName(String name) {
        int entryCount = getEntryCount();
        int output = 0;
        while (output < entryCount) {
            Account tempAccount = getAccount(output);
            if (tempAccount.getName().equals(name)) {
                return output;
            }
            output += 1;
        }
        return -1;
    }
    
    private int findEmptyEntry() {
        int entryCount = getEntryCount();
        int output = 0;
        while (output < entryCount) {
            Account tempAccount = getAccount(output);
            if (tempAccount == null) {
                return output;
            }
            output += 1;
        }
        return output;
    }
    
    public int getNextId() {
        int output = 0;
        int entryCount = getEntryCount();
        int index = 0;
        while (index < entryCount) {
            Account tempAccount = getAccount(index);
            int tempId = tempAccount.getId();
            if (tempId > output) {
                output = tempId;
            }
            index += 1;
        }
        output += 1;
        return output;
    }
    
    public void updateAccount(Account account) {
        int index = findAccountById(account.getId());
        if (index > -1) {
            setAccount(index, account);
        }
    }
    
    public void addAccount(Account account) {
        int index = findEmptyEntry();
        setAccount(index, account);
    }
    
    public void removeAccount(Account account) {
        int index = findAccountById(account.getId());
        if (index > -1) {
            setEntry(index, "[]");
        }
    }
    
    public Account getAccountById(int id) {
        int index = findAccountById(id);
        if (index > -1) {
            return getAccount(index);
        }
        return null;
    }
    
    public Account getAccountByName(String name) {
        int index = findAccountByName(name);
        if (index > -1) {
            return getAccount(index);
        }
        return null;
    }
}
