package com.mycompany.ostracraft;

import com.mycompany.ostracraft.blocks.*;
import com.mycompany.ostracraft.routes.CreateAccountRoute;
import com.mycompany.ostracraft.routes.ImageRoute;
import com.mycompany.ostracraft.routes.LoginRoute;
import com.mycompany.ostracraft.routes.PageRoute;
import com.mycompany.ostracraft.routes.WorldUpdateRoute;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import javax.swing.Timer;
import static spark.Spark.*;

public class App {
    
    //public static String rootPath = "/Users/johneisenmann/Dropbox/Ostracraft/";
    public static String rootPath = "./";
    public static List<BlockType> blockTypeSet = new ArrayList<BlockType>();
    public static List<Recipe> recipeSet = new ArrayList<Recipe>();
    public static long seed;
    public static int framesPerSecond = 10;
    public static final World world = new World();
    public static ChatEngine chatEngine = new ChatEngine();
    
    private static int timeSinceLastSave = 0;
    
    public static void main( String[] args ) {
        
        seed = Long.parseLong(SettingsDatabase.database.getSettingValue("seed"));
        
        System.out.println("SEED: " + seed);
        
        addBlockType(Air.blockType);
        addBlockType(Grass.blockType);
        addBlockType(Dirt.blockType);
        addBlockType(Tree.blockType);
        addBlockType(Sapling.blockType);
        addBlockType(Planks.blockType);
        addBlockType(Flower.blockType);
        addBlockType(BabyFlower.blockType);
        addBlockType(Stone.blockType);
        addBlockType(Bedrock.blockType);
        addBlockType(Coal.blockType);
        addBlockType(Iron.blockType);
        addBlockType(Diamond.blockType);
        addBlockType(Silicon.blockType);
        addBlockType(Torch.blockType);
        addBlockType(WoodenTool.blockType);
        addBlockType(StoneTool.blockType);
        addBlockType(IronTool.blockType);
        addBlockType(DiamondTool.blockType);
        addBlockType(WoodenArmor.blockType);
        addBlockType(IronArmor.blockType);
        addBlockType(DiamondArmor.blockType);
        addBlockType(Door.blockType);
        addBlockType(Wire.blockType);
        addBlockType(WireActive.blockType);
        addBlockType(OrInput.blockType);
        addBlockType(AndInput.blockType);
        addBlockType(BufferOutput.blockType);
        addBlockType(InvertingOutput.blockType);
        addBlockType(Switch.blockType);
        addBlockType(SwitchActive.blockType);
        addBlockType(Led.blockType);
        addBlockType(LedActive.blockType);
        addBlockType(Gun.blockType);
        addBlockType(Bullet.blockType);
        addBlockType(MinedStone.blockType);
        addBlockType(XorInput.blockType);
        addBlockType(DelayOutput.blockType);
        addBlockType(FlipFlop.blockType);
        addBlockType(FlipFlopActive.blockType);
        addBlockType(SetInput.blockType);
        addBlockType(ResetInput.blockType);
        addBlockType(DataInput.blockType);
        addBlockType(ClockInput.blockType);
        addBlockType(ToggleInput.blockType);
        addBlockType(CoalPowder.blockType);
        addBlockType(FlowerPetals.blockType);
        
        addRecipe(new Recipe(0,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Planks.blockType, 1)
                },
                new BlockTypeQuantityPair(Torch.blockType, 1),
                5.0));
        addRecipe(new Recipe(1,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Coal.blockType, 1),
                    new BlockTypeQuantityPair(Planks.blockType, 1)
                },
                new BlockTypeQuantityPair(Torch.blockType, 10),
                2.0));
        addRecipe(new Recipe(2,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Planks.blockType, 5)
                },
                new BlockTypeQuantityPair(WoodenTool.blockType, 1),
                5.0));
        addRecipe(new Recipe(3,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(MinedStone.blockType, 4),
                    new BlockTypeQuantityPair(Planks.blockType, 1)
                },
                new BlockTypeQuantityPair(StoneTool.blockType, 1),
                5.0));
        addRecipe(new Recipe(4,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Iron.blockType, 4),
                    new BlockTypeQuantityPair(Planks.blockType, 1)
                },
                new BlockTypeQuantityPair(IronTool.blockType, 1),
                5.0));
        addRecipe(new Recipe(5,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Diamond.blockType, 4),
                    new BlockTypeQuantityPair(Planks.blockType, 1)
                },
                new BlockTypeQuantityPair(DiamondTool.blockType, 1),
                5.0));
        addRecipe(new Recipe(6,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Planks.blockType, 20)
                },
                new BlockTypeQuantityPair(WoodenArmor.blockType, 1),
                10.0));
        addRecipe(new Recipe(7,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Iron.blockType, 20)
                },
                new BlockTypeQuantityPair(IronArmor.blockType, 1),
                10.0));
        addRecipe(new Recipe(8,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Diamond.blockType, 20)
                },
                new BlockTypeQuantityPair(DiamondArmor.blockType, 1),
                10.0));
        addRecipe(new Recipe(9,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Planks.blockType, 5)
                },
                new BlockTypeQuantityPair(Door.blockType, 1),
                5.0));
        addRecipe(new Recipe(10,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Iron.blockType, 1)
                },
                new BlockTypeQuantityPair(Wire.blockType, 20),
                2.0));
        addRecipe(new Recipe(11,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Silicon.blockType, 1)
                },
                new BlockTypeQuantityPair(OrInput.blockType, 5),
                2.0));
        addRecipe(new Recipe(12,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Silicon.blockType, 1)
                },
                new BlockTypeQuantityPair(AndInput.blockType, 5),
                2.0));
        addRecipe(new Recipe(13,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Silicon.blockType, 1)
                },
                new BlockTypeQuantityPair(BufferOutput.blockType, 5),
                2.0));
        addRecipe(new Recipe(14,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Silicon.blockType, 1)
                },
                new BlockTypeQuantityPair(InvertingOutput.blockType, 5),
                2.0));
        addRecipe(new Recipe(15,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(MinedStone.blockType, 1),
                    new BlockTypeQuantityPair(Planks.blockType, 1)
                },
                new BlockTypeQuantityPair(Switch.blockType, 1),
                2.0));
        addRecipe(new Recipe(16,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Diamond.blockType, 1),
                    new BlockTypeQuantityPair(Silicon.blockType, 1)
                },
                new BlockTypeQuantityPair(Led.blockType, 10),
                2.0));
        addRecipe(new Recipe(17,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Iron.blockType, 10)
                },
                new BlockTypeQuantityPair(Gun.blockType, 1),
                5.0));
        addRecipe(new Recipe(18,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Iron.blockType, 1)
                },
                new BlockTypeQuantityPair(Bullet.blockType, 10),
                2.0));
        addRecipe(new Recipe(19,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Silicon.blockType, 1)
                },
                new BlockTypeQuantityPair(XorInput.blockType, 5),
                2.0));
        addRecipe(new Recipe(20,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Silicon.blockType, 1)
                },
                new BlockTypeQuantityPair(DelayOutput.blockType, 5),
                2.0));
        addRecipe(new Recipe(21,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Iron.blockType, 1),
                    new BlockTypeQuantityPair(Silicon.blockType, 1)
                },
                new BlockTypeQuantityPair(FlipFlop.blockType, 5),
                2.0));
        addRecipe(new Recipe(22,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Silicon.blockType, 1)
                },
                new BlockTypeQuantityPair(SetInput.blockType, 5),
                2.0));
        addRecipe(new Recipe(23,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Silicon.blockType, 1)
                },
                new BlockTypeQuantityPair(ResetInput.blockType, 5),
                2.0));
        addRecipe(new Recipe(24,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Silicon.blockType, 1)
                },
                new BlockTypeQuantityPair(DataInput.blockType, 5),
                2.0));
        addRecipe(new Recipe(25,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Silicon.blockType, 1)
                },
                new BlockTypeQuantityPair(ClockInput.blockType, 5),
                2.0));
        addRecipe(new Recipe(26,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Silicon.blockType, 1)
                },
                new BlockTypeQuantityPair(ToggleInput.blockType, 5),
                2.0));
        addRecipe(new Recipe(27,
                new BlockTypeQuantityPair[]{
                    new BlockTypeQuantityPair(Coal.blockType, 1)
                },
                new BlockTypeQuantityPair(CoalPowder.blockType, 5),
                2.0));
        
        port(8080);
        get("/login", new PageRoute(rootPath + "pages/login.html", false));
        get("/loginAction", new LoginRoute());
        get("/createAccount", new PageRoute(rootPath + "pages/createAccount.html", false));
        get("/createAccountAction", new CreateAccountRoute());
        get("/game", new PageRoute(rootPath + "pages/game.html", true));
        get("/sprites.png", new ImageRoute(rootPath + "pages/sprites.png"));
        get("/worldUpdate", new WorldUpdateRoute());
        
        int delay = 1000;
        ActionListener taskPerformer = new App.AppActionListener();
        new Timer(delay, taskPerformer).start();
        
        Runtime.getRuntime().addShutdownHook(new Thread() {
            @Override
            public void run() {
                synchronized (App.world) {
                    persistEverything();
                }
            }
        });
    }
    
    private static void addBlockType(BlockType blockType) {
        int tempId = blockType.getId();
        int index = blockTypeSet.size();
        while (index <= tempId) {
            blockTypeSet.add(null);
            index += 1;
        }
        blockTypeSet.set(tempId, blockType);
    }
    
    private static void addRecipe(Recipe recipe) {
        int tempId = recipe.getId();
        int index = recipeSet.size();
        while (index <= tempId) {
            recipeSet.add(null);
            index += 1;
        }
        recipeSet.set(tempId, recipe);
    }
    
    public static void persistEverything() {
        world.saveAllChunks();
        world.saveAllPlayers();
        System.out.println("Saved everything.");
    }
    
    public static int getTimeSinceLastSave() {
        return timeSinceLastSave;
    }
    
    public static void setTimeSinceLastSave(int timeSinceLastSave) {
        App.timeSinceLastSave = timeSinceLastSave;
    }
    
    private static class AppActionListener implements ActionListener {
        public void actionPerformed(ActionEvent evt) {
            synchronized (App.world) {
                App.setTimeSinceLastSave(App.getTimeSinceLastSave() + 1);
                if (App.getTimeSinceLastSave() > 60) {
                    App.setTimeSinceLastSave(0);
                    App.persistEverything();
                }
            }
        }
    }
}

