/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.mycompany.ostracraft;

import com.mycompany.ostracraft.blocks.Air;
import com.mycompany.ostracraft.blocks.Bedrock;
import com.mycompany.ostracraft.blocks.BlockType;
import com.mycompany.ostracraft.blocks.Flower;
import com.mycompany.ostracraft.blocks.Grass;
import com.mycompany.ostracraft.blocks.LoadingBlock;
import com.mycompany.ostracraft.blocks.Stone;
import com.mycompany.ostracraft.blocks.Tree;
import com.mycompany.ostracraft.entities.AngryFace;
import com.mycompany.ostracraft.entities.CannonFace;
import com.mycompany.ostracraft.entities.Entity;
import com.mycompany.ostracraft.entities.LazyFace;
import com.mycompany.ostracraft.entities.Player;
import com.mycompany.ostracraft.entities.PoisonFace;
import com.mycompany.ostracraft.entities.PrettyFace;
import com.mycompany.ostracraft.entities.SadFace;
import com.mycompany.ostracraft.entities.SleepyFace;
import com.mycompany.ostracraft.entities.SpikyFace;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 *
 * @author Jack
 */
public class Chunk {
    
    public static int width = 64;
    public static int height = 2;
    
    public static int caveDensity1 = 3;
    public static int caveDensity2 = 5;
    public static int lightingSize = 8;
    
    private static int blockListSize = width * width * height;
    private static Random tickRandom = new Random();
    
    private Pos pos;
    private int[] blockIdList = new int[blockListSize];
    private int[] blockDataList = new int[blockListSize];
    private int[] lightingList = new int[blockListSize];
    private List<Entity> entityList = new ArrayList<Entity>();
    private boolean shouldBeUnloaded = false;
    private boolean isDisabled;
    
    public Chunk(Pos pos, boolean isDisabled) {
        this.pos = pos;
        this.isDisabled = isDisabled;
        
        //System.out.println(pos.toString());
        
        App.world.addChunk(this);
        
        if (ChunkDatabase.database.chunkExists(pos)) {
            ChunkDatabase.database.loadChunk(this);
        } else {
            generateTerrain();
        }
        
        if (!isDisabled) {
            generateLighting();
        }
    }
    
    private void generateLighting() {
        int index = 0;
        while (index < lightingList.length) {
            lightingList[index] = 0;
            index += 1;
        }
        Pos tempPos = new Pos(0, 0, 0);
        Pos tempOffset = new Pos(-lightingSize, -lightingSize, -1);
        while (tempOffset.getZ() < height + 1) {
            tempPos.set(pos);
            tempPos.add(tempOffset);
            // Load disabled chunks to avoid a chain reaction.
            BlockType tempBlockType = App.world.getBlockType(tempPos, World.ChunkGenerationMode.GENERATE_DISABLED);
            if (tempBlockType != null) {
                if (tempBlockType.getEmitsLight()) {
                    addLight(tempPos, 1);
                }
            }
            tempOffset.setX(tempOffset.getX() + 1);
            if (tempOffset.getX() >= width + lightingSize) {
                tempOffset.setX(-lightingSize);
                tempOffset.setY(tempOffset.getY() + 1);
                if (tempOffset.getY() >= width + lightingSize) {
                    tempOffset.setY(-lightingSize);
                    tempOffset.setZ(tempOffset.getZ() + 1);
                }
            }
        }
    }
    
    private void generateTerrain() {
        Random tempRandom = new Random(App.seed + convertPosToSeedOffset(pos));
        
        Pos tempOffset = new Pos(0, 0, 0);
        while (tempOffset.getY() < width) {
            int tempIndex1 = tempOffset.getX() + tempOffset.getY() * width;
            int tempIndex2 = tempIndex1 + width * width;
            if (pos.getZ() > 0) {
                blockIdList[tempIndex1] = Air.blockType.getId();
                blockIdList[tempIndex2] = Air.blockType.getId();
            } else if (pos.getZ() == 0) {
                blockIdList[tempIndex1] = Grass.blockType.getId();
                int tempNumber = tempRandom.nextInt(70);
                if (tempNumber == 1) {
                    blockIdList[tempIndex2] = Tree.blockType.getId();
                } else if (tempNumber == 2) {
                    blockIdList[tempIndex2] = Flower.blockType.getId();
                } else {
                    blockIdList[tempIndex2] = Air.blockType.getId();
                }
            } else {
                blockIdList[tempIndex1] = Bedrock.blockType.getId();
                if (tempOffset.getX() % 2 == 0 && tempOffset.getY() % 2 == 0) {
                    int tempBlockId;
                    int tempBlockId2;
                    if (tempRandom.nextInt(caveDensity1) < 1) {
                        tempBlockId = Stone.blockType.getId();
                        tempBlockId2 = Air.blockType.getId();
                    } else {
                        tempBlockId = Air.blockType.getId();
                        tempBlockId2 = Stone.blockType.getId();
                    }
                    if (tempRandom.nextInt(caveDensity2) < 1) {
                        blockIdList[tempIndex2] = tempBlockId2;
                    } else {
                        blockIdList[tempIndex2] = tempBlockId;
                    }
                    if (tempRandom.nextInt(caveDensity2) < 1) {
                        blockIdList[tempIndex2 + 1] = tempBlockId2;
                    } else {
                        blockIdList[tempIndex2 + 1] = tempBlockId;
                    }
                    if (tempRandom.nextInt(caveDensity2) < 1) {
                        blockIdList[tempIndex2 + width] = tempBlockId2;
                    } else {
                        blockIdList[tempIndex2 + width] = tempBlockId;
                    }
                    if (tempRandom.nextInt(caveDensity2) < 1) {
                        blockIdList[tempIndex2 + width + 1] = tempBlockId2;
                    } else {
                        blockIdList[tempIndex2 + width + 1] = tempBlockId;
                    }
                }
            }
            tempOffset.setX(tempOffset.getX() + 1);
            if (tempOffset.getX() >= width) {
                tempOffset.setX(0);
                tempOffset.setY(tempOffset.getY() + 1);
            }
        }
        
    }
    
    public void addLight(Pos pos, int offset) {
        Pos tempPos = new Pos(0, 0, 0);
        Pos tempOffset = new Pos(-lightingSize, -lightingSize, -1);
        while (tempOffset.getZ() <= 1) {
            tempPos.set(pos);
            tempPos.add(tempOffset);
            if (containsPos(tempPos)) {
                int index = convertPosToIndex(tempPos);
                lightingList[index] += offset;
            }
            tempOffset.setX(tempOffset.getX() + 1);
            if (tempOffset.getX() > lightingSize) {
                tempOffset.setX(-lightingSize);
                tempOffset.setY(tempOffset.getY() + 1);
                if (tempOffset.getY() > lightingSize) {
                    tempOffset.setY(-lightingSize);
                    tempOffset.setZ(tempOffset.getZ() + 1);
                }
            }
        }
    }
    
    public Pos getPos() {
        return pos;
    }
    
    public boolean containsPos(Pos pos) {
        return (pos.getX() >= this.pos.getX() && pos.getX() < this.pos.getX() + width
                && pos.getY() >= this.pos.getY() && pos.getY() < this.pos.getY() + width
                && pos.getZ() >= this.pos.getZ() && pos.getZ() < this.pos.getZ() + height);
    }
    
    private int convertPosToIndex(Pos pos) {
        return ((pos.getX() - this.pos.getX())
                + (pos.getY() - this.pos.getY()) * width
                + (pos.getZ() - this.pos.getZ()) * width * width);
    }
    
    public BlockType getBlockType(Pos pos) {
        if (!containsPos(pos)) {
            return null;
        }
        int index = convertPosToIndex(pos);
        int tempBlockId = blockIdList[index];
        return App.blockTypeSet.get(tempBlockId);
    }
    
    public void setBlockType(Pos pos, BlockType blockType) {
        if (isDisabled) {
            return;
        }
        if (!containsPos(pos)) {
            return;
        }
        int index = convertPosToIndex(pos);
        int tempBlockId = blockIdList[index];
        BlockType tempBlockType = App.blockTypeSet.get(tempBlockId);
        tempBlockId = blockType.getId();
        blockIdList[index] = tempBlockId;
        if (!tempBlockType.getEmitsLight() && blockType.getEmitsLight()) {
            addLight(pos, 1);
        }
        if (tempBlockType.getEmitsLight() && !blockType.getEmitsLight()) {
            addLight(pos, -1);
        }
    }
    
    public int getBlockData(Pos pos) {
        if (!containsPos(pos)) {
            return 0;
        }
        int index = convertPosToIndex(pos);
        return blockDataList[index];
    }
    
    public void setBlockData(Pos pos, int data) {
        if (isDisabled) {
            return;
        }
        if (!containsPos(pos)) {
            return;
        }
        int index = convertPosToIndex(pos);
        blockDataList[index] = data;
    }
    
    public boolean getLighting(Pos pos) {
        if (!containsPos(pos)) {
            return false;
        }
        int index = convertPosToIndex(pos);
        return (lightingList[index] > 0);
    }
    
    private static long convertPosToSeedOffset(Pos pos) {
        return (long)(32768 + pos.getX() / width)
                + ((long)(32768 + pos.getY() / width) << 16)
                + ((long)(32768 + pos.getZ() / height) << 32);
    }
    
    public void addEntity(Entity entity) {
        if (isDisabled) {
            return;
        }
        entityList.add(entity);
        entity.setChunk(this);
    }
    
    public void removeEntity(Entity entity) {
        if (isDisabled) {
            return;
        }
        entityList.remove(entity);
    }
    
    public Entity getEntity(Pos pos) {
        if (isDisabled) {
            return null;
        }
        int index = 0;
        while (index < entityList.size()) {
            Entity tempEntity = entityList.get(index);
            if (tempEntity.getPos().equals(pos)) {
                return tempEntity;
            }
            index += 1;
        }
        return null;
    }
    
    public List<Entity> getEntityList() {
        return new ArrayList<Entity>(entityList);
    }
    
    public boolean getShouldBeUnloaded() {
        return shouldBeUnloaded;
    }
    
    public void setShouldBeUnloaded(boolean shouldBeUnloaded) {
        this.shouldBeUnloaded = shouldBeUnloaded;
    }
    
    public boolean getIsDisabled() {
        return isDisabled;
    }
    
    public void setIsDisabled(boolean isDisabled) {
        if (this.isDisabled && !isDisabled) {
            generateLighting();
        }
        this.isDisabled = isDisabled;        
    }
    
    public void spawnMob() {
        Pos tempPos = pos.copy();
        tempPos.setX(tempPos.getX() + tickRandom.nextInt(width));
        tempPos.setY(tempPos.getY() + tickRandom.nextInt(width));
        tempPos.setZ(tempPos.getZ() + tickRandom.nextInt(height));
        Entity tempEntity = null;
        if (App.world.getLighting(tempPos, World.ChunkGenerationMode.DONT_GENERATE)) {
            if (tempPos.getZ() >= 0 && tempPos.getZ() <= 1) {
                int tempNumber = tickRandom.nextInt(2);
                if (tempNumber == 0) {
                    tempEntity = new LazyFace(tempPos);
                }
                if (tempNumber == 1) {
                    tempEntity = new SleepyFace(tempPos);
                }
            }
        } else {
            int tempNumber;
            if (tempPos.getZ() >= 0) {
                tempNumber = tickRandom.nextInt(6);
            } else {
                tempNumber = tickRandom.nextInt(5);
            }
            if (tempNumber == 0) {
                tempEntity = new PoisonFace(tempPos);
            }
            if (tempNumber == 1) {
                tempEntity = new AngryFace(tempPos);
            }
            if (tempNumber == 2) {
                tempEntity = new CannonFace(tempPos);
            }
            if (tempNumber == 3) {
                tempEntity = new SpikyFace(tempPos);
            }
            if (tempNumber == 4) {
                tempEntity = new SadFace(tempPos);
            }
            if (tempNumber == 5) {
                tempEntity = new PrettyFace(tempPos);
            }
        }
        if (tempEntity != null) {
            if (!App.world.getBlockType(tempPos, World.ChunkGenerationMode.DONT_GENERATE).getIsPhantom(tempEntity)) {
                return;
            }
            Pos tempPos2 = tempPos.copy();
            tempPos2.setZ(tempPos2.getZ() - 1);
            BlockType tempBlockType = App.world.getBlockType(tempPos2, World.ChunkGenerationMode.DONT_GENERATE);
            if (tempBlockType.getIsPhantom(tempEntity) && tempBlockType.getId() != LoadingBlock.blockType.getId()) {
                return;
            }
            App.world.addEntity(tempEntity);
        }
    }
    
    public void despawnMob() {
    	int index = tickRandom.nextInt(entityList.size());
    	Entity tempEntity = entityList.get(index);
    	if (!(tempEntity instanceof Player)) {
    		App.world.removeEntity(tempEntity);
    	}
    }
    
    public int[] getBlockIdList() {
        return blockIdList;
    }
    
    public int[] getBlockDataList() {
        return blockDataList;
    }
    
    public void tick() {
        if (isDisabled) {
            return;
        }
        Pos tempPos = new Pos(0, 0, 0);
        int tempCount = 0;
        while (tempCount < 200) {
            tempPos.set(pos);
            tempPos.setX(tempPos.getX() + tickRandom.nextInt(width));
            tempPos.setY(tempPos.getY() + tickRandom.nextInt(width));
            tempPos.setZ(tempPos.getZ() + tickRandom.nextInt(height));
            BlockType tempBlockType = getBlockType(tempPos);
            if (tempBlockType != null) {
                tempBlockType.randomTick(tempPos);
            }
            tempCount += 1;
        }
        if (entityList.size() < 8) {
            spawnMob();
        }
        if (entityList.size() > 10) {
            despawnMob();
        }
    }
    
}
