/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.mycompany.ostracraft;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author Jack
 */
public class ChunkDatabase {
    
    public static int clusterChunkSize = 5;
    public static int clusterWidth = Chunk.width * clusterChunkSize;
    public static int clusterHeight = Chunk.height * clusterChunkSize;
    
    private static String path = App.rootPath + "data/chunks/";
    
    public static ChunkDatabase database = new ChunkDatabase();
    
    public boolean chunkExists(Pos pos) {
        File tempFile = getChunkFile(pos);
        return tempFile.exists();
    }
    
    public void loadChunk(Chunk chunk) {
        //System.out.println(chunk.getPos());
        File tempFile = getChunkFile(chunk.getPos());
        try {
            FileInputStream tempStream = new FileInputStream(tempFile);
            BufferedReader tempReader = new BufferedReader(new InputStreamReader(tempStream));
            int[] tempBlockIdList = chunk.getBlockIdList();
            int[] tempBlockDataList = chunk.getBlockDataList();
            int tempBlockIdListIndex = 0;
            int tempBlockDataListIndex = 0;
            boolean shouldWriteBlockId = false;
            boolean shouldWriteBlockData = false;
            while (true) {
                String tempLine = tempReader.readLine();
                if (tempLine == null) {
                    break;
                }
                if (tempLine.length() > 0) {
                    String[] tempList = tempLine.split(",");
                    String tempCommand = tempList[0];
                    if (tempCommand.equals("blockIdList")) {
                        shouldWriteBlockId = true;
                        shouldWriteBlockData = false;
                    } else if (tempCommand.equals("blockDataList")) {
                        shouldWriteBlockId = false;
                        shouldWriteBlockData = true;
                    } else {
                        int tempIndex = 0;
                        while (tempIndex < tempList.length) {
                            int tempQuantity = Integer.parseInt(tempList[tempIndex]);
                            int tempNumber = Integer.parseInt(tempList[tempIndex + 1]);
                            while (tempQuantity > 0) {
                                if (shouldWriteBlockId) {
                                    tempBlockIdList[tempBlockIdListIndex] = tempNumber;
                                    tempBlockIdListIndex += 1;
                                }
                                if (shouldWriteBlockData) {
                                    tempBlockDataList[tempBlockDataListIndex] = tempNumber;
                                    tempBlockDataListIndex += 1;
                                }
                                tempQuantity -= 1;
                            }
                            tempIndex += 2;
                        }
                    }
                }
            }
            tempReader.close();
        } catch (IOException exception) {
            System.out.println("Unable to access file.");
        }
    }
    
    public void saveChunk(Chunk chunk) {
        File tempFile = getChunkFile(chunk.getPos());
        try {
            FileOutputStream tempStream = new FileOutputStream(tempFile);
            BufferedWriter tempWriter = new BufferedWriter(new OutputStreamWriter(tempStream));
            int[] tempBlockIdList = chunk.getBlockIdList();
            int[] tempBlockDataList = chunk.getBlockDataList();
            int tempLength = Chunk.width * 2;
            tempWriter.write("blockIdList\n");
            List<String> tempLineList = new ArrayList<String>();
            int index = 0;
            while (index < tempBlockIdList.length) {
                int tempNumber = tempBlockIdList[index];
                index += 1;
                int tempQuantity = 1;
                while (index < tempBlockIdList.length) {
                    int tempNumber2 = tempBlockIdList[index];
                    if (tempNumber2 != tempNumber) {
                        break;
                    }
                    tempQuantity += 1;
                    index += 1;
                }
                tempLineList.add(Integer.toString(tempQuantity));
                tempLineList.add(Integer.toString(tempNumber));
                if (tempLineList.size() >= tempLength || index >= tempBlockIdList.length) {
                    StringBuilder tempBuilder = new StringBuilder();
                    tempBuilder.append(String.join(",", tempLineList));
                    tempBuilder.append("\n");
                    tempWriter.write(tempBuilder.toString());
                    tempLineList = new ArrayList<String>();
                }
            }
            tempWriter.write("blockDataList\n");
            tempLineList = new ArrayList<String>();
            index = 0;
            while (index < tempBlockDataList.length) {
                int tempNumber = tempBlockDataList[index];
                index += 1;
                int tempQuantity = 1;
                while (index < tempBlockDataList.length) {
                    int tempNumber2 = tempBlockDataList[index];
                    if (tempNumber2 != tempNumber) {
                        break;
                    }
                    tempQuantity += 1;
                    index += 1;
                }
                tempLineList.add(Integer.toString(tempQuantity));
                tempLineList.add(Integer.toString(tempNumber));
                if (tempLineList.size() >= tempLength || index >= tempBlockDataList.length) {
                    StringBuilder tempBuilder = new StringBuilder();
                    tempBuilder.append(String.join(",", tempLineList));
                    tempBuilder.append("\n");
                    tempWriter.write(tempBuilder.toString());
                    tempLineList = new ArrayList<String>();
                }
            }
            tempWriter.close();
        } catch (IOException exception) {
            System.out.println("Unable to access file.");
        }
    }
    
    private Pos convertPosToClusterPos(Pos pos) {
        Pos output = pos.copy();
        output.setX(World.floorDivide(output.getX(), clusterWidth) * clusterWidth);
        output.setY(World.floorDivide(output.getY(), clusterWidth) * clusterWidth);
        output.setZ(World.floorDivide(output.getZ(), clusterHeight) * clusterHeight);
        return output;
    }
    
    private File getChunkFile(Pos pos) {
        File tempDirectory = new File(path);
        if (!tempDirectory.exists()) {
            tempDirectory.mkdir();
        }
        Pos tempPos = convertPosToClusterPos(pos);
        String tempName = tempPos.getX() + "_" + tempPos.getY() + "_" + tempPos.getZ();
        String tempPath = path + tempName;
        tempDirectory = new File(tempPath);
        if (!tempDirectory.exists()) {
            tempDirectory.mkdir();
        }
        tempName = pos.getX() + "_" + pos.getY() + "_" + pos.getZ() + ".txt";
        tempPath = tempPath + "/" + tempName;
        return new File(tempPath);
    }
    
}
