/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.mycompany.ostracraft;

import com.mycompany.ostracraft.blocks.BlockType;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 *
 * @author Jack
 */
public class Inventory {
    
    private int accountId;
    private List<InventoryItem> inventoryItemList;
    private InventoryItem armor;
    private boolean shouldSendToPlayer = false;
    private Random random = new Random();
    
    public Inventory(int accountId) {
        this.accountId = accountId;
        inventoryItemList = new ArrayList<InventoryItem>();
        armor = null;
    }
    
    public int getAccountId() {
        return accountId;
    }
    
    public List<InventoryItem> getInventoryItemList() {
        return new ArrayList<InventoryItem>(inventoryItemList);
    }
    
    public InventoryItem getInventoryItemById(int id) {
        int index = 0;
        while (index < inventoryItemList.size()) {
            InventoryItem tempInventoryItem = inventoryItemList.get(index);
            if (tempInventoryItem.getId() == id) {
                return tempInventoryItem;
            }
            index += 1;
        }
        return null;
    }
    
    public InventoryItem getInventoryItemByBlockId(int blockId) {
        int index = 0;
        while (index < inventoryItemList.size()) {
            InventoryItem tempInventoryItem = inventoryItemList.get(index);
            if (tempInventoryItem.getBlockId() == blockId) {
                return tempInventoryItem;
            }
            index += 1;
        }
        return null;
    }
    
    public int getNextInventoryItemId() {
        int output = 0;
        int index = 0;
        while (index < inventoryItemList.size()) {
            InventoryItem tempInventoryItem = inventoryItemList.get(index);
            int tempId = tempInventoryItem.getId();
            if (tempId >= output) {
                output = tempId + 1;
            }
            index += 1;
        }
        return output;
    }
    
    public void addInventoryItem(InventoryItem inventoryItem) {
        boolean shouldAddInventoryItem = true;
        BlockType tempBlockType = inventoryItem.getBlockType();
        if (tempBlockType.isStackable()) {
            InventoryItem tempInventoryItem = getInventoryItemByBlockId(tempBlockType.getId());
            if (tempInventoryItem != null) {
                increaseInventoryItemQuantity(tempInventoryItem, inventoryItem.getQuantity());
                shouldAddInventoryItem = false;
            }
        }
        if (shouldAddInventoryItem) {
            InventoryItem tempInventoryItem = getInventoryItemById(inventoryItem.getId());
            if (tempInventoryItem != null) {
                inventoryItem.setId(getNextInventoryItemId());
            }
            inventoryItemList.add(inventoryItem);
        }
    }
    
    public void removeInventoryItem(InventoryItem inventoryItem) {
        inventoryItemList.remove(inventoryItem);
    }
    
    public void increaseInventoryItemQuantity(InventoryItem inventoryItem, int quantity) {
        if (quantity < 0) {
            return;
        }
        inventoryItem.setQuantity(inventoryItem.getQuantity() + quantity);
    }
    
    public boolean decreaseInventoryItemQuantity(InventoryItem inventoryItem, int quantity) {
        if (quantity < 0) {
            return false;
        }
        int tempQuantity = inventoryItem.getQuantity() - quantity;
        if (tempQuantity < 0) {
            return false;
        }
        if (tempQuantity == 0) {
            removeInventoryItem(inventoryItem);
        } else {
            inventoryItem.setQuantity(tempQuantity);
        }
        return true;
    }
    
    public boolean canCraftRecipe(Recipe recipe) {
        BlockTypeQuantityPair[] tempList = recipe.getIngredientList();
        int index = 0;
        while (index < tempList.length) {
            BlockTypeQuantityPair tempPair = tempList[index];
            InventoryItem tempInventoryItem = getInventoryItemByBlockId(tempPair.getBlockType().getId());
            if (tempInventoryItem == null) {
                return false;
            }
            if (tempInventoryItem.getQuantity() < tempPair.getQuantity()) {
                return false;
            }
            index += 1;
        }
        return true;
    }
    
    public InventoryItem getArmor() {
        return armor;
    }
    
    public void setArmor(InventoryItem armor) {
        this.armor = armor;
    }
    
    public boolean getShouldSendToPlayer() {
        return shouldSendToPlayer;
    }
    
    public void setShouldSendToPlayer(boolean shouldSendToPlayer) {
        this.shouldSendToPlayer = shouldSendToPlayer;
    }
    
    public List<InventoryItem> removeRandomInventoryItems() {
        List<InventoryItem> output = new ArrayList<InventoryItem>();
        int index = inventoryItemList.size() - 1;
        while (index >= 0 ) {
            InventoryItem tempInventoryItem = inventoryItemList.get(index);
            if (random.nextInt(8) == 1) {
                removeInventoryItem(tempInventoryItem);
                output.add(tempInventoryItem);
            }
            index -= 1;
        }
        return output;
    }
    
}
