/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.mycompany.ostracraft;

import com.mycompany.ostracraft.blocks.BlockType;
import com.mycompany.ostracraft.blocks.LoadingBlock;
import com.mycompany.ostracraft.entities.Entity;
import com.mycompany.ostracraft.entities.Player;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import javax.swing.Timer;
import signals.Signal;

/**
 *
 * @author Jack
 */
public class World {
    
    public static enum ChunkGenerationMode {
        DONT_GENERATE, GENERATE_ENABLED, GENERATE_DISABLED
    }
    public static Pos[] electronicSignalOffsetSet = {
        new Pos(-1, 0, 0),
        new Pos(1, 0, 0),
        new Pos(0, -1, 0),
        new Pos(0, 1, 0),
        new Pos(0, 0, -1),
        new Pos(0, 0, 1),
    };
    
    private Map<Pos, Chunk> chunkMap = new HashMap<Pos, Chunk>();
    private List<Chunk> chunkList = new ArrayList<Chunk>();
    private List<Entity> entityList = new ArrayList<Entity>();
    private List<Player> playerList = new ArrayList<Player>();
    private List<Signal> signalList = new ArrayList<Signal>();
    private Random random = new Random();
    
    private int chunkLoadWidth = 256;
    private int chunkLoadHeight = 10;
    private int chunkLoadDelay = 0;
    
    private long dayLength = App.framesPerSecond * 20 * 60;
    private long time = 0;
    //private long time = dayLength / 2 + 10;
    
    public World() {
        int delay = 1000 / App.framesPerSecond;
        ActionListener taskPerformer = new WorldActionListener();
        new Timer(delay, taskPerformer).start();
    }
    
    // number2 should be positive.
    public static int floorDivide(int number1, int number2) {
        if (number1 >= 0) {
            return number1 / number2;
        } else {
            return (number1 - number2 + 1) / number2;
        }
    }
    
    private Pos convertPosToChunkPos(Pos pos) {
        Pos output = pos.copy();
        output.setX(floorDivide(output.getX(), Chunk.width) * Chunk.width);
        output.setY(floorDivide(output.getY(), Chunk.width) * Chunk.width);
        output.setZ(floorDivide(output.getZ(), Chunk.height) * Chunk.height);
        return output;
    }
    
    public boolean chunkExists(Pos pos) {
        Pos tempPos = convertPosToChunkPos(pos);
        return chunkMap.containsKey(tempPos);
    }
    
    // BE VERY CAREFUL ABOUT CALLING THIS.
    public void addChunk(Chunk chunk) {
        chunkMap.put(chunk.getPos(), chunk);
        chunkList.add(chunk);
    }
    
    // generationMode values:
    // 0 = Don't generate
    // 1 = Generate enabled
    // 2 = Generate disabled
    public Chunk getChunk(Pos pos, ChunkGenerationMode generationMode) {
        Pos tempPos = convertPosToChunkPos(pos);
        Chunk output = chunkMap.get(tempPos);
        if (output == null && generationMode != ChunkGenerationMode.DONT_GENERATE) {
            output = new Chunk(tempPos, generationMode == ChunkGenerationMode.GENERATE_DISABLED);
        }
        return output;
    }
    
    public BlockType getBlockType(Pos pos, ChunkGenerationMode generationMode) {
        Chunk tempChunk = getChunk(pos, generationMode);
        if (tempChunk == null) {
            return LoadingBlock.blockType;
        }
        return tempChunk.getBlockType(pos);
    }
    
    public void setBlockType(Pos pos, BlockType blockType, ChunkGenerationMode generationMode) {
        Chunk tempChunk = getChunk(pos, generationMode);
        if (tempChunk == null) {
            return;
        }
        tempChunk.setBlockType(pos, blockType);
        blockType.performSpawnAction(pos);
    }
    
    public int getBlockData(Pos pos, ChunkGenerationMode generationMode) {
        Chunk tempChunk = getChunk(pos, generationMode);
        if (tempChunk == null) {
            return 0;
        }
        return tempChunk.getBlockData(pos);
    }
    
    public void setBlockData(Pos pos, int data, ChunkGenerationMode generationMode) {
        Chunk tempChunk = getChunk(pos, generationMode);
        if (tempChunk == null) {
            return;
        }
        tempChunk.setBlockData(pos, data);
    }
    
    public boolean getLighting(Pos pos, ChunkGenerationMode generationMode) {
        Chunk tempChunk = getChunk(pos, generationMode);
        if (tempChunk == null) {
            return false;
        }
        if (pos.getZ() >= 0 && !getIsNight()) {
            return true;
        }
        return tempChunk.getLighting(pos);
    }
    
    public boolean getIsLightOrAboveGround(Pos pos, ChunkGenerationMode generationMode) {
        if (pos.getZ() >= 0) {
            return true;
        }
        return getLighting(pos, generationMode);
    }
    
    public void addEntity(Entity entity) {
        Chunk tempChunk = getChunk(entity.getPos(), ChunkGenerationMode.GENERATE_ENABLED);
        tempChunk.addEntity(entity);
        entityList.add(entity);
        if (entity instanceof Player) {
            playerList.add((Player)entity);
        }
    }
    
    public void removeEntity(Entity entity) {
        Chunk tempChunk = getChunk(entity.getPos(), ChunkGenerationMode.GENERATE_ENABLED);
        if (tempChunk != null) {
            tempChunk.removeEntity(entity);
        }
        entity.setHasBeenRemoved(true);
        entityList.remove(entity);
        if (entity instanceof Player) {
            Player tempPlayer = (Player)entity;
            tempPlayer.save();
            playerList.remove(tempPlayer);
        }
    }
    
    public Entity getEntity(Pos pos) {
        Chunk tempChunk = getChunk(pos, ChunkGenerationMode.DONT_GENERATE);
        if (tempChunk == null) {
            return null;
        }
        return tempChunk.getEntity(pos);
    }
    
    public Entity getEntityById(int id) {
        int index = 0;
        while (index < entityList.size()) {
            Entity tempEntity = entityList.get(index);
            if (tempEntity.getId() == id) {
                return tempEntity;
            }
            index += 1;
        }
        return null;
    }
    
    public Player getPlayerByAccountId(int accountId) {
        int index = 0;
        while (index < entityList.size()) {
            Entity tempEntity = entityList.get(index);
            if (tempEntity instanceof Player) {
                Player tempPlayer = (Player)tempEntity;
                if (tempPlayer.getAccount().getId() == accountId) {
                    return tempPlayer;
                }
            }
            index += 1;
        }
        return null;
    }
    
    public int getNextEntityId() {
        int output = 0;
        int index = 0;
        while (index < entityList.size()) {
            Entity tempEntity = entityList.get(index);
            int tempId = tempEntity.getId();
            if (tempId >= output) {
                output = tempId + 1;
            }
            index += 1;
        }
        return output;
    }
    
    public List<Entity> getEntitiesInRegion(Pos pos, int width, int height) {
        List<Entity> output = new ArrayList<Entity>();
        int index = 0;
        while (index < entityList.size()) {
            Entity tempEntity = entityList.get(index);
            Pos tempPos = tempEntity.getPos();
            if (tempPos.getX() >= pos.getX() && tempPos.getX() < pos.getX() + width
                    && tempPos.getY() >= pos.getY() && tempPos.getY() < pos.getY() + width
                    && tempPos.getZ() >= pos.getZ() && tempPos.getZ() < pos.getZ() + height) {
                output.add(tempEntity);
            }
            index += 1;
        }
        return output;
    }
    
    public List<Player> getPlayersInRegion(Pos pos, int width, int height) {
        List<Player> output = new ArrayList<Player>();
        int index = 0;
        while (index < playerList.size()) {
            Player tempPlayer = playerList.get(index);
            Pos tempPos = tempPlayer.getPos();
            if (tempPos.getX() >= pos.getX() && tempPos.getX() < pos.getX() + width
                    && tempPos.getY() >= pos.getY() && tempPos.getY() < pos.getY() + width
                    && tempPos.getZ() >= pos.getZ() && tempPos.getZ() < pos.getZ() + height) {
                output.add(tempPlayer);
            }
            index += 1;
        }
        return output;
    }
    
    public List<Player> getPlayers() {
        return new ArrayList<Player>(playerList);
    }
    
    public void addSignal(Signal signal) {
        signalList.add(signal);
    }
    
    public void generateExplosion(Pos pos, int radius) {
        Pos tempPos = pos.copy();
        Pos tempOffset = new Pos(-radius, -radius, -radius);
        tempPos.add(tempOffset);
        List<Entity> tempEntityList = getEntitiesInRegion(tempPos, radius * 2 + 1, radius * 2 + 1);
        int index = 0;
        while (index < tempEntityList.size()) {
            Entity tempEntity = tempEntityList.get(index);
            tempEntity.damage(30.0, null);
            index += 1;
        }
        while (tempOffset.getZ() <= radius) {
            tempPos.set(pos);
            tempPos.add(tempOffset);
            if (random.nextInt(2) == 1) {
                BlockType tempBlockType = getBlockType(tempPos, ChunkGenerationMode.DONT_GENERATE);
                tempBlockType.respondToExplosion(tempPos);
            }
            tempOffset.setX(tempOffset.getX() + 1);
            if (tempOffset.getX() > radius) {
                tempOffset.setX(-radius);
                tempOffset.setY(tempOffset.getY() + 1);
                if (tempOffset.getY() > radius) {
                    tempOffset.setY(-radius);
                    tempOffset.setZ(tempOffset.getZ() + 1);
                }
            }
        }
    }
    
    private Chunk loadChunk(Pos pos) {
        Chunk output = getChunk(pos, ChunkGenerationMode.GENERATE_ENABLED);
        output.setIsDisabled(false);
        return output;
    }
    
    private void unloadChunk(Chunk chunk) {
        List<Entity> tempEntityList = chunk.getEntityList();
        int index = 0;
        while (index < tempEntityList.size()) {
            Entity tempEntity = tempEntityList.get(index);
            if (!(tempEntity instanceof Player)) {
                entityList.remove(tempEntity);
            }
            index += 1;
        }
        ChunkDatabase.database.saveChunk(chunk);
        chunkMap.remove(chunk.getPos());
        chunkList.remove(chunk);
    }
    
    public void saveAllChunks() {
        int index = 0;
        while (index < chunkList.size()) {
            Chunk tempChunk = chunkList.get(index);
            ChunkDatabase.database.saveChunk(tempChunk);
            index += 1;
        }
    }
    
    public void saveAllPlayers() {
        int index = 0;
        while (index < playerList.size()) {
            Player tempPlayer = playerList.get(index);
            tempPlayer.save();
            index += 1;
        }
    }
    
    public boolean getIsNight() {
        return time % dayLength > dayLength / 2;
    }
    
    public long getTime() {
        return time;
    }
    
    private class WorldActionListener implements ActionListener {
        public void actionPerformed(ActionEvent evt) {
            synchronized (App.world) {
                chunkLoadDelay -= 1;
                if (chunkLoadDelay <= 0) {
                    int index = chunkList.size() - 1;
                    while (index >= 0) {
                        Chunk tempChunk = chunkList.get(index);
                        tempChunk.setShouldBeUnloaded(true);
                        index -= 1;
                    }
                    index = entityList.size() - 1;
                    while (index >= 0) {
                        Entity tempEntity = entityList.get(index);
                        if (tempEntity instanceof Player) {
                            Pos tempPos = new Pos(0, 0, 0);
                            Pos tempOffset = new Pos(0, 0, 0);
                            while (tempOffset.getZ() < chunkLoadHeight) {
                                tempPos.set(tempEntity.getPos());
                                tempPos.setX(tempPos.getX() - chunkLoadWidth / 2);
                                tempPos.setY(tempPos.getY() - chunkLoadWidth / 2);
                                tempPos.setZ(tempPos.getZ() - chunkLoadHeight / 2);
                                tempPos.add(tempOffset);
                                Chunk tempChunk = loadChunk(tempPos);
                                tempChunk.setShouldBeUnloaded(false);
                                tempOffset.setX(tempOffset.getX() + Chunk.width);
                                if (tempOffset.getX() >= chunkLoadWidth) {
                                    tempOffset.setX(0);
                                    tempOffset.setY(tempOffset.getY() + Chunk.width);
                                    if (tempOffset.getY() >= chunkLoadWidth) {
                                        tempOffset.setY(0);
                                        tempOffset.setZ(tempOffset.getZ() + Chunk.height);
                                    }
                                }
                            }
                        }
                        index -= 1;
                    }
                    index = chunkList.size() - 1;
                    while (index >= 0) {
                        Chunk tempChunk = chunkList.get(index);
                        if (tempChunk.getShouldBeUnloaded()) {
                            unloadChunk(tempChunk);
                        }
                        index -= 1;
                    }
                    //System.out.println("ENTITY COUNT: " + entityList.size());
                    chunkLoadDelay = App.framesPerSecond;
                }

                int index = chunkList.size() - 1;
                while (index >= 0) {
                    Chunk tempChunk = chunkList.get(index);
                    tempChunk.tick();
                    index -= 1;
                }

                List<Entity> tempList = new ArrayList<Entity>(entityList);
                index = tempList.size() - 1;
                while (index >= 0) {
                    Entity tempEntity = tempList.get(index);
                    if (!tempEntity.getHasBeenRemoved()) {
                    	tempEntity.tick();
                    }
                    index -= 1;
                }

                List<Signal> tempWaitingSignalList = new ArrayList<Signal>();
                int tempCount = 0;
                while (signalList.size() > 0 && tempCount < 1000) {
                    List<Signal> tempSignalList = signalList;
                    signalList = new ArrayList<Signal>();
                    index = 0;
                    while (index < tempSignalList.size()) {
                        Signal tempSignal = tempSignalList.get(index);
                        int tempDelay = tempSignal.getDelay();
                        if (tempDelay <= 0) {
                            tempSignal.performAction();
                        } else {
                            tempSignal.setDelay(tempDelay - 1);
                            tempWaitingSignalList.add(tempSignal);
                        }
                        index += 1;
                    }
                    tempCount += 1;
                }
                signalList.addAll(tempWaitingSignalList);

                time += 1;
            }
        }
    };
    
}

