/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.mycompany.ostracraft;

import com.mycompany.ostracraft.blocks.BlockType;
import com.mycompany.ostracraft.blocks.LoadingBlock;
import com.mycompany.ostracraft.entities.Entity;
import com.mycompany.ostracraft.entities.Player;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author Jack
 */
public class WorldUpdateCommandGenerator {
    
    public static WorldUpdateCommandGenerator generator = new WorldUpdateCommandGenerator();
    
    public List<String> generateBlocksCommand(Pos pos, int width, int height) {
        List<String> output = new ArrayList<String>();
        output.add("blocks");
        output.add(Integer.toString(pos.getX()));
        output.add(Integer.toString(pos.getY()));
        output.add(Integer.toString(pos.getZ()));
        output.add(Integer.toString(width));
        output.add(Integer.toString(height));
        Pos tempPos2 = new Pos(0, 0, 0);
        Pos tempOffset = new Pos(0, 0, 0);
        while (tempOffset.getZ() < height) {
            tempPos2.set(pos);
            tempPos2.add(tempOffset);
            BlockType tempBlockType = App.world.getBlockType(tempPos2, World.ChunkGenerationMode.GENERATE_ENABLED);
            if (tempBlockType == null) {
                output.add(Integer.toString(LoadingBlock.blockType.getId()));
            } else {
                output.add(Integer.toString(tempBlockType.getId()));
            }
            tempOffset.setX(tempOffset.getX() + 1);
            if (tempOffset.getX() >= width) {
                tempOffset.setX(0);
                tempOffset.setY(tempOffset.getY() + 1);
                if (tempOffset.getY() >= width) {
                    tempOffset.setY(0);
                    tempOffset.setZ(tempOffset.getZ() + 1);
                }
            }
        }
        return output;
    }
    
    public List<String> generateEntityCommand(Entity entity) {
        List<String> output = new ArrayList<String>();
        output = new ArrayList<String>();
        output.add("entity");
        output.add(Integer.toString(entity.getTypeId()));
        output.add(Integer.toString(entity.getId()));
        output.add(Integer.toString(entity.getPos().getX()));
        output.add(Integer.toString(entity.getPos().getY()));
        output.add(Integer.toString(entity.getPos().getZ()));
        int tempBlockId = -1;
        if (entity instanceof Player) {
            Player tempPlayer = (Player)entity;
            InventoryItem tempArmor = tempPlayer.getInventory().getArmor();
            if (tempArmor != null) {
                tempBlockId = tempArmor.getBlockId();
            }
        }
        output.add(Integer.toString(tempBlockId));
        return output;
    }
    
    public List<String> generateReportInventoryCommand(InventoryItem inventoryItem) {
        List<String> output = new ArrayList<String>();
        output.add("createInventory");
        output.add(Integer.toString(inventoryItem.getId()));
        output.add(Integer.toString(inventoryItem.getBlockId()));
        output.add(Integer.toString(inventoryItem.getBlockData()));
        output.add(Integer.toString(inventoryItem.getQuantity()));
        return output;
    }
    
    public List<String> generateCreateInventoryCommand(Inventory inventory, InventoryItem inventoryItem) {
        inventory.addInventoryItem(inventoryItem);
        return generateReportInventoryCommand(inventoryItem);
    }
    
    public List<String> generateIncreaseInventoryCommand(Inventory inventory, InventoryItem inventoryItem, int quantity) {
        inventory.increaseInventoryItemQuantity(inventoryItem, quantity);
        List<String> output = new ArrayList<String>();
        output.add("increaseInventory");
        output.add(Integer.toString(inventoryItem.getId()));
        output.add(Integer.toString(quantity));
        return output;
    }
    
    public List<String> generateCreateOrIncreaseInventoryCommand(Inventory inventory, BlockType blockType, int blockData, int quantity) {
        boolean shouldCreateInventoryItem = true;
        if (blockType.isStackable()) {
            InventoryItem tempInventoryItem = inventory.getInventoryItemByBlockId(blockType.getId());
            if (tempInventoryItem != null) {
                shouldCreateInventoryItem = false;
                return WorldUpdateCommandGenerator.generator.generateIncreaseInventoryCommand(inventory, tempInventoryItem, quantity);
            }
        }
        if (shouldCreateInventoryItem) {
            int tempId = inventory.getNextInventoryItemId();
            InventoryItem tempInventoryItem = new InventoryItem(tempId, blockType.getId(), blockData, quantity);
            return WorldUpdateCommandGenerator.generator.generateCreateInventoryCommand(inventory, tempInventoryItem);
        }
        return null;
    }
    
    public List<String> generateDecreaseInventoryCommand(Inventory inventory, InventoryItem inventoryItem, int quantity) {
        inventory.decreaseInventoryItemQuantity(inventoryItem, quantity);
        List<String> output = new ArrayList<String>();
        output.add("decreaseInventory");
        output.add(Integer.toString(inventoryItem.getId()));
        output.add(Integer.toString(quantity));
        return output;
    }
    
    public List<String> generateUpdateInventoryDataCommand(InventoryItem inventoryItem) {
        List<String> output = new ArrayList<String>();
        output.add("updateInventoryData");
        output.add(Integer.toString(inventoryItem.getId()));
        output.add(Integer.toString(inventoryItem.getBlockData()));
        return output;
    }
    
    public List<String> generateClearInventoryCommand() {
        List<String> output = new ArrayList<String>();
        output.add("clearInventory");
        return output;
    }
    
    public List<List<String>> generateRetrieveEntireInventoryCommand(Inventory inventory) {
        List<List<String>> output = new ArrayList<List<String>>();
        List<String> tempOutputCommand = WorldUpdateCommandGenerator.generator.generateClearInventoryCommand();
        output.add(tempOutputCommand);
        List<InventoryItem> inventoryItemList = inventory.getInventoryItemList();
        int index = 0;
        while (index < inventoryItemList.size()) {
            InventoryItem tempInventoryItem = inventoryItemList.get(index);
            tempOutputCommand = WorldUpdateCommandGenerator.generator.generateReportInventoryCommand(tempInventoryItem);
            output.add(tempOutputCommand);
            index += 1;
        }
        tempOutputCommand = generateReportArmorCommand(inventory.getArmor());
        output.add(tempOutputCommand);
        return output;
    }
    
    public List<String> generateIsNightCommand() {
        List<String> output = new ArrayList<String>();
        output.add("isNight");
        if (App.world.getIsNight()) {
            output.add("1");
        } else {
            output.add("0");
        }
        return output;
    }
    
    public List<String> generatePosCommand(Pos pos) {
        List<String> output = new ArrayList<String>();
        output = new ArrayList<String>();
        output.add("pos");
        output.add(Integer.toString(pos.getX()));
        output.add(Integer.toString(pos.getY()));
        output.add(Integer.toString(pos.getZ()));
        return output;
    }
    
    public List<String> generateDeleteArmorCommand(Inventory inventory) {
        inventory.setArmor(null);
        List<String> output = generateReportArmorCommand(inventory.getArmor());
        return output;
    }
    
    public List<List<String>> generateRemoveArmorCommand(Inventory inventory) {
        List<List<String>> output = new ArrayList<List<String>>();
        InventoryItem tempInventoryItem = inventory.getArmor();
        if (tempInventoryItem == null) {
            return output;
        }
        List<String> tempOutputCommand = generateDeleteArmorCommand(inventory);
        output.add(tempOutputCommand);
        tempOutputCommand = generateCreateOrIncreaseInventoryCommand(inventory, tempInventoryItem.getBlockType(), tempInventoryItem.getBlockData(), 1);
        output.add(tempOutputCommand);
        return output;
    }
    
    public List<String> generateReportArmorCommand(InventoryItem inventoryItem) {
        List<String> output = new ArrayList<String>();
        if (inventoryItem == null) {
            output.add("deleteArmor");
        } else {
            output.add("createArmor");
            output.add(Integer.toString(inventoryItem.getId()));
            output.add(Integer.toString(inventoryItem.getBlockId()));
            output.add(Integer.toString(inventoryItem.getBlockData()));
            output.add(Integer.toString(inventoryItem.getQuantity()));
        }
        return output;
    }
    
    public List<List<String>> generateCreateArmorCommand(Inventory inventory, InventoryItem inventoryItem) {
        List<List<String>> output;
        InventoryItem tempOldArmor = inventory.getArmor();
        if (tempOldArmor != null) {
            output = generateRemoveArmorCommand(inventory);
        } else {
            output = new ArrayList<List<String>>();
        }
        List<String> tempOutputCommand;
        tempOutputCommand = generateDecreaseInventoryCommand(inventory, inventoryItem, 1);
        output.add(tempOutputCommand);
        inventoryItem.setQuantity(1);
        inventory.setArmor(inventoryItem);
        tempOutputCommand = generateReportArmorCommand(inventoryItem);
        output.add(tempOutputCommand);
        return output;
    }
    
    public List<String> generateUpdateArmorCommand(Inventory inventory) {
        List<String> output = new ArrayList<String>();
        output.add("updateArmor");
        output.add(Integer.toString(inventory.getArmor().getBlockData()));
        return output;
    }
    
    public List<String> generateHealthCommand(double health) {
        List<String> output = new ArrayList<String>();
        output.add("health");
        output.add(Integer.toString((int)health));
        return output;
    }
    
    public List<String> generateOnlinePlayersCommand() {
        List<String> output = new ArrayList<String>();
        output.add("onlinePlayers");
        List<Player> tempPlayerList = App.world.getPlayers();
        int index = 0;
        while (index < tempPlayerList.size()) {
            Player tempPlayer = tempPlayerList.get(index);
            String tempName = tempPlayer.getAccount().getName();
            output.add(tempName);
            index += 1;
        }
        return output;
    }
    
    public List<String> generateChatCommand(Player player) {
        List<String> tempMessageList = App.chatEngine.getMessages(player.getChatMessageId());
        if (tempMessageList.size() < 1) {
            return null;
        }
        List<String> output = new ArrayList<String>();
        output.add("chat");
        output.addAll(tempMessageList);
        player.setChatMessageId(App.chatEngine.getId());
        return output;
    }
    
}
