/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.mycompany.ostracraft.entities;

import com.mycompany.ostracraft.App;
import com.mycompany.ostracraft.Chunk;
import com.mycompany.ostracraft.InventoryItem;
import com.mycompany.ostracraft.Pos;
import com.mycompany.ostracraft.World;
import com.mycompany.ostracraft.blocks.BlockType;
import com.mycompany.ostracraft.blocks.Tool;
import java.util.List;
import java.util.Random;

/**
 *
 * @author Jack
 */
public abstract class Entity {
    
    private int id;
    private Pos pos;
    private Chunk chunk;
    private double health;
    private int attackDelay = 0;
    private boolean hasBeenRemoved = false;
    private static Random random = new Random();
    
    public Entity(Pos pos) {
        this.id = App.world.getNextEntityId();
        this.pos = pos;
        this.health = 100;
    }

    public abstract int getTypeId();
    
    public int getId() {
        return id;
    }
    
    public Pos getPos() {
        return pos;
    }
    
    public Random getRandom() {
        return random;
    }
    
    public Chunk getChunk() {
        return chunk;
    }
    
    public void setChunk(Chunk chunk) {
        this.chunk = chunk;
    }
    
    public boolean getHasBeenRemoved() {
    	return hasBeenRemoved;
    }
    
    public void setHasBeenRemoved(boolean hasBeenRemoved) {
    	this.hasBeenRemoved = hasBeenRemoved;
    }
    
    public double getHealth() {
        return health;
    }
    
    public void setHealth(double health) {
        this.health = health;
        if (this.health <= 0.0) {
            die(null);
        }
    }
    
    public void heal(double offset) {
        health += offset;
        if (health > 100.0) {
            health = 100.0;
        }
    }
    
    public void damage(double offset, Entity entity) {
        health -= offset;
        if (health <= 0.0) {
            die(entity);
        }
    }
    
    public void die(Entity entity) {
        App.world.removeEntity(this);
    }
    
    public double getAttackDamage(InventoryItem inventoryItem) {
        return 10.0;
    }
    
    public boolean damageInventoryItem(InventoryItem inventoryItem) {
        if (inventoryItem != null) {
            if (inventoryItem.getBlockType() instanceof Tool) {
                inventoryItem.setBlockData(inventoryItem.getBlockData() + 1);
                return true;
            }
        }
        return false;
    }
    
    public void attackPlayerAtOffset(Pos offset) {
        if (attackDelay > 0) {
            return;
        }
        Pos tempPos = pos.copy();
        tempPos.add(offset);
        Entity tempEntity = App.world.getEntity(tempPos);
        if (tempEntity != null) {
            if (tempEntity instanceof Player) {
                tempEntity.damage(getAttackDamage(null), this);
                attackDelay = App.framesPerSecond / 2;
            }
        }
    }
    
    public void attackNeighborPlayers() {
        attackPlayerAtOffset(new Pos(-1, 0, 0));
        attackPlayerAtOffset(new Pos(1, 0, 0));
        attackPlayerAtOffset(new Pos(0, -1, 0));
        attackPlayerAtOffset(new Pos(0, 1, 0));
        attackPlayerAtOffset(new Pos(0, 0, -1));
        attackPlayerAtOffset(new Pos(0, 0, 1));
    }
    
    public void walkRandomly() {
        int tempDirection = random.nextInt(4);
        if (tempDirection == 0) {
            walk(new Pos(-1, 0, 0));
        } else if (tempDirection == 1) {
            walk(new Pos(1, 0, 0));
        } else if (tempDirection == 2) {
            walk(new Pos(0, -1, 0));
        } else if (tempDirection == 3) {
            walk(new Pos(0, 1, 0));
        }
    }
    
    public void walkTowardPos(Pos pos, int minimumDistance) {
        Pos tempPos = pos.copy();
        tempPos.subtract(this.pos);
        int tempDistanceX = Math.abs(tempPos.getX());
        int tempDistanceY = Math.abs(tempPos.getY());
        if (tempDistanceX + tempDistanceY > minimumDistance) {
            if (tempDistanceX > tempDistanceY) {
                if (tempPos.getX() > 0) {
                    walk(new Pos(1, 0, 0));
                } else {
                    walk(new Pos(-1, 0, 0));
                }
            } else {
                if (tempPos.getY() > 0) {
                    walk(new Pos(0, 1, 0));
                } else {
                    walk(new Pos(0, -1, 0));
                }
            }
        }
    }
    
    public Entity getClosestEntity(List<Entity> entityList) {
        Entity output = null;
        int closestDistance = 9999999;
        int index = 0;
        while (index < entityList.size()) {
            Entity tempEntity = entityList.get(index);
            Pos tempPos = tempEntity.getPos();
            int tempDistance = pos.getSquareDistance(tempPos);
            if (tempDistance < closestDistance) {
                output = tempEntity;
                closestDistance = tempDistance;
            }
            index += 1;
        }
        return output;
    }
    
    public Pos getClosestPos(List<Pos> posList) {
        Pos output = null;
        int closestDistance = 9999999;
        int index = 0;
        while (index < posList.size()) {
            Pos tempPos = posList.get(index);
            int tempDistance = pos.getSquareDistance(tempPos);
            if (tempDistance < closestDistance) {
                output = tempPos;
                closestDistance = tempDistance;
            }
            index += 1;
        }
        return output;
    }
    
    public World.ChunkGenerationMode getChunkGenerationMode() {
        return World.ChunkGenerationMode.DONT_GENERATE;
    }
    
    public boolean move(Pos offset) {
        Pos tempPos = pos.copy();
        tempPos.add(offset);
        BlockType tempBlockType = App.world.getBlockType(tempPos, getChunkGenerationMode());
        if (tempBlockType != null) {
            if (tempBlockType.getIsPhantom(this)) {
                Entity tempEntity = App.world.getEntity(tempPos);
                if (tempEntity != null) {
                    return false;
                }
                Chunk tempChunk = App.world.getChunk(tempPos, getChunkGenerationMode());
                if (tempChunk == null) {
                    App.world.removeEntity(this);
                } else {
                    if (chunk != null) {
                        chunk.removeEntity(this);
                    }
                    pos.set(tempPos);
                    tempChunk.addEntity(this);
                }
                return true;
            }
        }
        return false;
    }
    
    public void fall() {
        while (true) {
            boolean tempResult = move(new Pos(0, 0, -1));
            if (!tempResult) {
                return;
            }
        }
    }
    
    public void walk(Pos offset) {
        boolean tempResult = move(offset);
        if (!tempResult) {
            Pos tempPos = pos.copy();
            tempPos.setZ(tempPos.getZ() + 1);
            BlockType tempBlockType = App.world.getBlockType(tempPos, getChunkGenerationMode());
            if (tempBlockType != null) {
                if (tempBlockType.getIsPhantom(this)) {
                    Pos tempOffset = offset.copy();
                    tempOffset.setZ(tempOffset.getZ() + 1);
                    move(tempOffset);
                }
            }
        }
        this.fall();
    }
    
    public void tick() {
        if (attackDelay >= 0) {
            attackDelay -= 1;
        }
        
    }
    
}
