/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.mycompany.ostracraft.entities;

import com.mycompany.ostracraft.Account;
import com.mycompany.ostracraft.App;
import com.mycompany.ostracraft.Inventory;
import com.mycompany.ostracraft.InventoryDatabase;
import com.mycompany.ostracraft.InventoryItem;
import com.mycompany.ostracraft.PlayerDatabase;
import com.mycompany.ostracraft.Pos;
import com.mycompany.ostracraft.World;
import com.mycompany.ostracraft.blocks.Armor;
import com.mycompany.ostracraft.blocks.BlockType;
import com.mycompany.ostracraft.blocks.Tool;
import java.util.List;
import java.util.Random;

/**
 *
 * @author Jack
 */
public class Player extends FallingEntity {
    
    public static int spawnRange = 64;
    
    private Account account;
    private Inventory inventory;
    private long lastUpdateTime = App.world.getTime();
    private double maximumTimeBudget = 20.0;
    private double timeBudget = 5.0;
    private boolean armorHasChanged = false;
    private int chatMessageId;
    private static Random random = new Random();
    
    public Player(Pos pos, Account account) {
        super(pos);
        this.account = account;
        this.inventory = InventoryDatabase.database.loadInventory(account.getId());
        // TODO: Load player state.
        this.chatMessageId = App.chatEngine.getId() - 10;
    }
    
    @Override
    public int getTypeId() {
        return 0;
    }
    
    @Override
    public World.ChunkGenerationMode getChunkGenerationMode() {
        return World.ChunkGenerationMode.GENERATE_ENABLED;
    }
    
    @Override
    public void damage(double offset, Entity entity) {
        InventoryItem tempInventoryItem = inventory.getArmor();
        if (tempInventoryItem != null) {
            BlockType tempBlockType = tempInventoryItem.getBlockType();
            if (tempBlockType instanceof Armor) {
                Armor tempArmor = (Armor)tempBlockType;
                offset /= tempArmor.getDefense();
                tempInventoryItem.setBlockData(tempInventoryItem.getBlockData() + 1);
                if (tempBlockType.shouldRemoveFromInventory(tempInventoryItem.getBlockData())) {
                    inventory.setArmor(null);
                }
                armorHasChanged = true;
            }
        }
        super.damage(offset, entity);
    }
    
    @Override
    public void die(Entity entity) {
        Inventory tempInventory = getInventory();
        List<InventoryItem> tempInventoryItemList = tempInventory.removeRandomInventoryItems();
            if (tempInventoryItemList.size() > 0) {
            tempInventory.setShouldSendToPlayer(true);
            if (entity != null) {
                if (entity instanceof Player) {
                    Player tempPlayer = (Player)entity;
                    Inventory tempInventory2 = tempPlayer.getInventory();
                    int index = 0;
                    while (index < tempInventoryItemList.size()) {
                        InventoryItem tempInventoryItem = tempInventoryItemList.get(index);
                        tempInventory2.addInventoryItem(tempInventoryItem);
                        index += 1;
                    }
                    tempInventory2.setShouldSendToPlayer(true);
                }
            }
        }
        setHealth(100.0);
        Pos tempOffset = new Pos(random.nextInt(spawnRange), random.nextInt(spawnRange), 5);
        tempOffset.subtract(getPos());
        move(tempOffset);
    }
    
    @Override
    public double getAttackDamage(InventoryItem inventoryItem) {
        if (inventoryItem != null) {
            BlockType tempBlockType = inventoryItem.getBlockType();
            if (tempBlockType instanceof Tool) {
                Tool tempTool = (Tool)tempBlockType;
                return 10.0 * tempTool.getEfficiency();
            }
        }
        return 10.0;
    }
    
    public Account getAccount() {
        return account;
    }
    
    public Inventory getInventory() {
        return inventory;
    }
    
    public void activityUpdate() {
        lastUpdateTime = App.world.getTime();
    }
    
    public long getTimeSinceLastUpdate() {
        return App.world.getTime() - lastUpdateTime;
    }
    
    public void useTime(double time) {
        if (timeBudget > -2.0) {
            timeBudget -= time;
        }
    }
    
    public boolean isInTimeDebt() {
        return (timeBudget < 0.0);
    }
    
    public boolean getArmorHasChanged() {
        return armorHasChanged;
    }
    
    public void setArmorHasChanged(boolean armorHasChanged) {
        this.armorHasChanged = armorHasChanged;
    }
    
    public int getChatMessageId() {
        return chatMessageId;
    }
    
    public void setChatMessageId(int chatMessageId) {
        this.chatMessageId = chatMessageId;
    }
    
    public void save() {
        PlayerDatabase.database.savePlayer(this);
    }
    
    @Override
    public void tick() {
        super.tick();
        
        if (getTimeSinceLastUpdate() > App.framesPerSecond * 8) {
            App.world.removeEntity(this);
        }
        
        if (timeBudget < maximumTimeBudget) {
            timeBudget += 1.0 / App.framesPerSecond;
        }
        
        /*
        if (App.world.getTime() % App.framesPerSecond == 0) {
            System.out.println("TIME BUDGET: " + timeBudget);
        }
        */
    }
    
}
