/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package signals;

import com.mycompany.ostracraft.App;
import com.mycompany.ostracraft.Pos;
import com.mycompany.ostracraft.World;
import com.mycompany.ostracraft.blocks.BlockType;
import com.mycompany.ostracraft.blocks.ElectronicBlock;
import com.mycompany.ostracraft.blocks.ElectronicInput;
import com.mycompany.ostracraft.blocks.ElectronicOutput;
import com.mycompany.ostracraft.blocks.InvertingOutput;
import com.mycompany.ostracraft.blocks.Led;
import com.mycompany.ostracraft.blocks.LedActive;
import com.mycompany.ostracraft.blocks.Wire;
import com.mycompany.ostracraft.blocks.WireActive;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author Jack
 */
public class ElectronicSignal implements Signal {
    
    public static enum ElectronicMessage {
        SET, RESET, CLOCK, TOGGLE
    }
    
    private Pos pos;
    private boolean value;
    int delay;
    ElectronicMessage message = null;
    
    public ElectronicSignal(Pos pos, boolean value) {
        this.pos = pos;
        this.value = value;
        this.delay = 0;
    }
    
    public ElectronicSignal(Pos pos, boolean value, int delay) {
        this.pos = pos;
        this.value = value;
        this.delay = delay;
    }
    
    public Pos getPos() {
        return pos;
    }
    
    public boolean getValue() {
        return value;
    }
    
    public int getDelay() {
        return delay;
    }

    public void setDelay(int delay) {
        this.delay = delay;
    }
    
    public ElectronicMessage getMessage() {
        return message;
    }

    public void setMessage(ElectronicMessage message) {
        this.message = message;
    }
    
    public void performAction() {
        BlockType tempBlockType = App.world.getBlockType(pos, World.ChunkGenerationMode.DONT_GENERATE);
        if (tempBlockType instanceof ElectronicBlock) {
            ElectronicBlock tempElectronicBlock = (ElectronicBlock)tempBlockType;
            tempElectronicBlock.processSignal(this);
        }
    }
    
    public void propogateChannel(boolean value) {
        int index = 0;
        while (index < World.electronicSignalOffsetSet.length) {
            Pos tempPos = pos.copy();
            Pos tempOffset = World.electronicSignalOffsetSet[index];
            tempPos.add(tempOffset);
            BlockType tempBlockType = App.world.getBlockType(tempPos, World.ChunkGenerationMode.DONT_GENERATE);
            if (tempBlockType instanceof ElectronicBlock) {
                ElectronicBlock tempElectronicBlock = (ElectronicBlock)tempBlockType;
                if (tempElectronicBlock.canAcceptChannel(tempPos)) {
                    App.world.addSignal(new ElectronicSignal(tempPos, value));
                }
            }
            index += 1;
        }
    }
    
    public void propogateInput(boolean value, ElectronicMessage message) {
        int index = 0;
        while (index < World.electronicSignalOffsetSet.length) {
            Pos tempPos = pos.copy();
            Pos tempOffset = World.electronicSignalOffsetSet[index];
            tempPos.add(tempOffset);
            BlockType tempBlockType = App.world.getBlockType(tempPos, World.ChunkGenerationMode.DONT_GENERATE);
            if (tempBlockType instanceof ElectronicBlock) {
                ElectronicBlock tempElectronicBlock = (ElectronicBlock)tempBlockType;
                if (tempElectronicBlock.canAcceptInput(tempPos)) {
                    ElectronicSignal tempSignal = new ElectronicSignal(tempPos, value);
                    tempSignal.setMessage(message);
                    App.world.addSignal(tempSignal);
                }
            }
            index += 1;
        }
    }
    
    public void propogateOutput(boolean value) {
        propogateOutputWithDelay(value, 0);
    }
    
    public void propogateOutputWithDelay(boolean value, int delay) {
        int index = 0;
        while (index < World.electronicSignalOffsetSet.length) {
            Pos tempPos = pos.copy();
            Pos tempOffset = World.electronicSignalOffsetSet[index];
            tempPos.add(tempOffset);
            BlockType tempBlockType = App.world.getBlockType(tempPos, World.ChunkGenerationMode.DONT_GENERATE);
            if (tempBlockType instanceof ElectronicBlock) {
                ElectronicBlock tempElectronicBlock = (ElectronicBlock)tempBlockType;
                if (tempElectronicBlock.canAcceptOutput(tempPos)) {
                    App.world.addSignal(new ElectronicSignal(tempPos, value, delay));
                }
            }
            index += 1;
        }
    }
    
    public void propogateModule(boolean value) {
        int index = 0;
        while (index < World.electronicSignalOffsetSet.length) {
            Pos tempPos = pos.copy();
            Pos tempOffset = World.electronicSignalOffsetSet[index];
            tempPos.add(tempOffset);
            BlockType tempBlockType = App.world.getBlockType(tempPos, World.ChunkGenerationMode.DONT_GENERATE);
            if (tempBlockType instanceof ElectronicBlock) {
                ElectronicBlock tempElectronicBlock = (ElectronicBlock)tempBlockType;
                if (tempElectronicBlock.canAcceptModule(tempPos)) {
                    App.world.addSignal(new ElectronicSignal(tempPos, value));
                }
            }
            index += 1;
        }
    }

}
